@@ -6,9 +6,10 @@ import typer
 from loguru import logger
 from pathlib import Path
 
-from .constants import FileFormat, Language, Category
+from .constants import FileFormat, Language, Description, Heirarchy
 from .catalog import Catalog
 
+
 cli = typer.Typer()
 
 
@@ -23,13 +24,13 @@ def main(
         show_default=True,
         help="Choose catalog language",
     ),
-    category: Category = typer.Option(
-        Category.AllDisciplines,
-        "--category",
-        "-C",
+    description: Description = typer.Option(
+        Description.All_Disciplines,
+        "--description",
+        "-D",
         show_default=True,
         show_choices=True,
-        help="Choose a catalog catagory.",
+        help="Choose a catalog description.",
     ),
 ):
     """Springer Textbook Bulk Download Tool
@@ -77,51 +78,179 @@ def main(
     #     it to the typer.Context object using the attribute 'obj'.
 
     try:
-        ctx.obj = Catalog(language, category)
+        ctx.obj = Catalog(language, description)
     except KeyError as error:
         value = error.args[0]
         typer.secho(f"Failed to locate a catalog for '{value}'", fg="red")
         raise typer.Exit(-1)
 
 
-@cli.command()
-def list(
+@cli.command(name="list")
+def list_subcommand(
     ctx: typer.Context,
-    file_format: FileFormat = typer.Option(
-        FileFormat.pdf, "--format", "-f", show_default=True, show_choices=True
-    ),
+    heirarchy: Heirarchy,
+    match: str = typer.Option(None, "--match", "-m", help="String used for matching."),
     long_format: bool = typer.Option(
-        False, "--long-format", "-l", help="Show more information for each book.",
+        False,
+        "--long-format",
+        "-l",
+        is_flag=True,
+        show_default=True,
+        help="Display selected information in a longer format.",
     ),
 ):
-    """List titles of textbooks in the catalog.
+    """List books, package, packages, catalog or catalogs,
 
     Examples
     
-    List titles available in the default catalog (en-all):
+    List titles available in the default catalog:
 
-    `$ springer list`
+    `$ springer list books`
+
+    List packages available in the default catalog:
+
+    `$ springer list packages`
 
     List titles available in the German language, all disciplines catalog:
 
-    `$ springer --language de --category all list`
+    `$ springer --language de --description all list books`
+
+    List all eBook packages in the default catalog:
+
+    `$ springer list packages`
+
+    List all eBook packages in the default catalog whose name match:
+
+    `$ springer list package -m science`
+
+    List information about the current catalog:
+
+    `$ springer list catalog`
+
+    List information about the Germal language, Emergency Nursing catalog:
+
+    `$ springer --language de --description med list catalog`
+
+    
 
     """
 
-    for index, textbook in enumerate(ctx.obj.textbooks(file_format), start=1):
-        print(f"  Title #{index:03d}: ", textbook.title)
+    if heirarchy == Heirarchy.Books:
+        ctx.obj.list_books(long_format)
+        return
+
+    if heirarchy is Heirarchy.Package:
+        if match:
+            for name, package in ctx.obj.packages.items():
+                if match.casefold() in name.casefold():
+                    ctx.obj.list_package(name, package, long_format)
+                    return
+        else:
+            heirarchy = Heirarchy.Packages
+
+    if heirarchy is Heirarchy.Packages:
+        ctx.obj.list_packages(long_format)
+        return
+
+    if heirarchy is Heirarchy.Catalog:
+        catalogs = [ctx.obj]
+
+    if heirarchy is Heirarchy.Catalogs:
+        catalogs = Catalog.all_catalogs()
+
+    for catalog in catalogs:
+        print(catalog)
         if long_format:
-            print("        Path> ", textbook.path)
-            print("      Author> ", textbook.author)
-            print("     Edition> ", textbook.edition)
-            print("        ISBN> ", textbook.isbn)
-            print("       EISBN> ", textbook.eisbn)
-            print("     DOI URL> ", textbook.doi_url)
+            list_subcommand(ctx, Heirarchy.Packages, long_format)
+        print("\N{OCTAGONAL SIGN}")
 
 
-@cli.command()
-def download(
+@cli.command(name="refresh")
+def refresh_subcommand(
     ctx: typer.Context,
+    catalog_url: str = typer.Option(
+        None, "--url", "-u", help="URL for Excel-formatted catalog"
+    ),
+    all_catalogs: bool = typer.Option(False, "--all", is_flag=True),
+):
+    """Refresh the cached catalog of Springer textbooks.
+
+    If --all is specified, the --url option is ignored.
+
+    Examples
+
+    Update English language catalog:
+
+    `$ springer --language en refresh`
+
+    Update German language catalog whose description is 'all':
+
+    `$ springer --language de --description all refresh`
+
+    Update German language catalog whose description is 'med' with a new URL:
+
+    `$ springer -L de -D med refresh --url https://example.com/api/endpoint/something/v11`
+
+    Update all catalogs:
+
+    `$ springer refresh --all`
+
+    """
+
+    if not all_catalogs:
+        ctx.obj.fetch_catalog(catalog_url)
+        print(ctx.obj)
+        return
+
+    for catalog in Catalog.all_catalogs():
+        catalog.fetch_catalog()
+        print(catalog)
+
+
+@cli.command(name="clean")
+def clean_subcommand(
+    ctx: typer.Context,
+    force: bool = typer.Option(False, "--force", "-F", is_flag=True),
+    all_catalogs: bool = typer.Option(False, "--all", is_flag=True),
+):
+    """Removes the cached catalog.
+
+    Examples
+
+    Remove the English language, all disciplines cached catalog:
+
+    `$ springer clean --force`
+
+    Remove the German language emergency nursing cached catalog:
+
+    `$ springer -L de -D med clean --force`
+
+    Remove all catalogs:
+    
+    `$ springer clean --force --all`
+    """
+
+    if not force:
+        typer.secho("The --force switch is required!", fg="red")
+        raise typer.Exit(-1)
+
+    if not all_catalogs:
+        ctx.obj.cache_file.unlink()
+        return
+
+    for catalog in Catalog.all_catalogs():
+        catalog.cache_file.unlink()
+
+
+@cli.command(name="download")
+def download_subcommand(
+    ctx: typer.Context,
+    package: str = typer.Option(
+        None, "--package-name", "-p", help="Package name to match (partial name OK)."
+    ),
+    file_format: FileFormat = typer.Option(
+        FileFormat.pdf, "--format", "-f", show_default=True, show_choices=True
+    ),
     dest_path: Path = typer.Option(
         Path.cwd(),
         "--dest-path",
@@ -129,9 +258,6 @@ def download(
         show_default=True,
         help="Destination directory for downloaded files.",
     ),
-    file_format: FileFormat = typer.Option(
-        FileFormat.pdf, "--format", "-f", show_default=True, show_choices=True
-    ),
     overwrite: bool = typer.Option(
         False,
         "--over-write",
@@ -159,7 +285,7 @@ def download(
     root directory where files will be stored. Each catalog will save 
     it's textbooks to:
     
-    dest_path/language/category/book_file_name.fmt
+    dest_path/language/description/book_file_name.fmt
 
     Files that fail to download will be logged to a file named:
 
@@ -187,13 +313,17 @@ def download(
 
     `$ springer download --dest-path pdfs --over-write`
 
-    Download all books in PDF from the German all disciplines catalog:
+    Download all books in PDF from the German/All_Disciplines catalog:
     
-    `$ springer -L de -C all download --dest-path german/all/pdfs`
+    `$ springer -L de -D all download --dest-path german/all/pdfs`
 
     Download all books from all catelogs in epub format:
 
-    `$ springer download --all --dest-path books --format epub`
+    `$ springer download --all --format epub`
+
+    Download all books in the 'Computer Science' package in pdf format:
+
+    `$ springer download -p Computer`
     """
 
     logger.configure(
@@ -208,121 +338,15 @@ def download(
         }
     )
 
-    if not all_catalogs:
-        ctx.obj.download(dest_path, file_format, overwrite=overwrite)
-        return
-
-    for language in Language:
-        for category in Category:
-            try:
-                catalog = Catalog(language, category)
-            except KeyError:
-                continue
-            dest = dest_path / language.value / category.value
-            catalog.download(dest, file_format, overwrite=overwrite)
-
-
-@cli.command()
-def refresh(
-    ctx: typer.Context,
-    catalog_url: str = typer.Option(
-        None, "--url", "-u", help="URL for Excel-formatted catalog"
-    ),
-    all_catalogs: bool = typer.Option(False, "--all", is_flag=True),
-):
-    """Refresh the cached catalog of Springer textbooks.
-
-    If --all is specified, the --url option is ignored.
-
-    Examples
-
-    Update English language catalog:
-
-    `$ springer --language en refresh`
-
-    Update German language catalog whose category is 'all':
-
-    `$ springer --language de --category all refresh`
-
-    Update German language catalog whose category is 'med' with a new URL:
-
-    `$ springer -L de -C med refresh --url https://example.com/api/endpoint/something/v11`
-
-    Update all catalogs:
-
-    `$ springer refresh --all`
-
-    """
+    dest_path = dest_path.resolve()
 
     if not all_catalogs:
-        ctx.obj.fetch_catalog(catalog_url)
+        if package:
+            ctx.obj.download_package(package, dest_path, file_format, overwrite)
+            return
+        ctx.obj.download(dest_path, file_format, overwrite)
         return
 
-    for language in Language:
-        for category in Category:
-            try:
-                catalog = Catalog(language, category)
-                catalog.fetch_catalog()
-                print(catalog)
-
-            except KeyError:
-                pass
-
-
-@cli.command()
-def clean(
-    ctx: typer.Context,
-    force: bool = typer.Option(False, "--force", "-F"),
-    all_catalogs: bool = typer.Option(False, "--all", is_flag=True),
-):
-    """Removes the cached catalog.
-
-    Examples
-
-    Remove the English language, all disciplines cached catalog:
-
-    `$ springer clean -F`
-
-    Remove the German language emergency nursing cached catalog:
-
-    `$ springer -L de -C med clean -F`
-    """
-
-    if not force:
-        typer.secho("The --force switch is required!", fg="red")
-        raise typer.Exit(-1)
-
-    if not all_catalogs:
-        ctx.obj.cache_file.unlink()
-        return
-
-    for language in Language:
-        for category in Category:
-            try:
-                Catalog(language, category).cache_file.unlink()
-            except KeyError:
-                pass
-
-
-@cli.command()
-def catalogs():
-    """List available catalogs.
-
-    Prints an entry for each known catalog:
-    
-    \b
-    - Catalog URL
-    - Language
-    - Category
-    - Cache file path
-    - Number of books in the catalog.
-
-    """
-
-    for language in Language:
-        for category in Category:
-            try:
-                catalog = Catalog(language, category)
-                print(catalog)
-            except KeyError:
-                pass
+    for catalog in Catalog.all_catalogs():
+        dest = dest_path / catalog.language.name / catalog.description.value
+        catalog.download(dest, file_format, overwrite=overwrite)
